/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2006 The LS-Universe Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#include "stdafx.h"
#include "Utility.h"

bool IsOS(const int &os)
{	
	static int osversion = 0;

	static bool bInitialized = false;
	
	if (!bInitialized)
	{
		BOOL bOsVersionInfoEx;
		OSVERSIONINFOEX verInfo;
		ZeroMemory(&verInfo, sizeof(OSVERSIONINFOEX));
		verInfo.dwOSVersionInfoSize = sizeof(OSVERSIONINFOEX);

		if( !(bOsVersionInfoEx = GetVersionEx ((OSVERSIONINFO *) &verInfo)) )
		{
			  verInfo.dwOSVersionInfoSize = sizeof (OSVERSIONINFO);
			  if (! GetVersionEx ( (OSVERSIONINFO *) &verInfo) ) 
			     return FALSE;
		}

		switch (verInfo.dwPlatformId)
		{
			// Test for the Windows NT product family.
			case VER_PLATFORM_WIN32_NT:

				osversion |= OS_NT;
			 
				// Test for the specific product.
				if ( verInfo.dwMajorVersion == 5 && verInfo.dwMinorVersion == 2 )
				{
					osversion |= OS_2KXP;
					osversion |= OS_XP;
				}

				if ( verInfo.dwMajorVersion == 5 && verInfo.dwMinorVersion == 1 )
				{
					osversion |= OS_2KXP;
					osversion |= OS_XP;
				}

				if ( verInfo.dwMajorVersion == 5 && verInfo.dwMinorVersion == 0 )
				{
					osversion |= OS_2KXP;
					osversion |= OS_2K;
				}

				if ( verInfo.dwMajorVersion <= 4 )
				{
					osversion |= OS_NT4;
				}

			 break;

			// Test for the Windows Me/98/95.
			case VER_PLATFORM_WIN32_WINDOWS:

				osversion |= OS_9X;

				if (verInfo.dwMajorVersion == 4 && verInfo.dwMinorVersion == 0)
				{
					osversion |= OS_95;
				} 

				if (verInfo.dwMajorVersion == 4 && verInfo.dwMinorVersion == 10)
				{
					osversion |= OS_98;
				} 

				if (verInfo.dwMajorVersion == 4 && verInfo.dwMinorVersion == 90)
				{
					osversion |= OS_ME;
				} 

				break;

			default:
				break;
	   }

		bInitialized = true;
	}

	return (osversion & os) ? true : false;
}

// retrieve an integer value from step.rc, make sure it is between minVal and maxVal inclusive
int GetxRCInt(const string &uniqueKey, const string &baseName, int defaultVal, int minVal, int maxVal)
{
	string key = uniqueKey + baseName;
	int answer = GetRCInt(key.c_str(), defaultVal);
	return max(min(answer, maxVal), minVal);
}

// retrieve a line from step.rc
string GetxRCLine(const string &uniqueKey, const string &baseName, const string &defaultVal)
{
	string key = uniqueKey + baseName;
	char buffer[MAX_LINE_LENGTH] = {0};
	GetRCLine(key.c_str(), buffer, MAX_LINE_LENGTH, defaultVal.c_str());

	return (string) buffer;
}

// retrieve a string value from step.rc
string GetxRCString(const string &uniqueKey, const string &baseName, const string &defaultVal)
{
	string key = uniqueKey + baseName;
	char buffer[MAX_LINE_LENGTH] = {0};
	GetRCString(key.c_str(), buffer, defaultVal.c_str(), MAX_LINE_LENGTH);
	return string(buffer);
}


void PrefixedLSSetVariable(string prefix, string var, string value)
{
	if ( !var.empty() )
		prefix.append(var);

	LSSetVariable(prefix.c_str(), value.c_str());
}

string IntToString(int input)
{
	char buffer[32] = {0};
	itoa( input, buffer, 10 );
	return (string) buffer;
}