/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2006 The LS-Universe Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#include "stdafx.h"
#include "Utility.h"
#include "VerInfo.h"

#include "TrashFunctions.h"

#define TIMER_TRASHCHECK	1000

#define TIMER_SHUTDOWN		1001
#define TIMER_PRESHUTDOWN	1002

#define TIMER_RECYCLE		1003

//Trash Interface
//----------------------------------------------------
string BecomeEmpty = GetxRCLine("xTrash", "BecomesEmpty", "");
string BecomeFilled = GetxRCLine("xTrash", "BecomesFilled", "");
string Change = GetxRCLine("xTrash", "OnChange", "");

int TrashTimeout = GetxRCInt("xTrash", "CheckInterval", 60, 1, 3600);

int oldtrashCount = -1;
string oldtrashSize = "";
bool isTrashEmpty = false;

bool isTrashSizeAvailable = false;
void CheckTrash(bool init = false);

void BangEmptyTrash(HWND hwndCaller, LPCSTR pszArgs);

int m_iHour = 0;
int m_iMinute = 0;
int m_iSecond = 0;
bool m_bExactTime = false;

bool m_bTimerFirstStart = true;

// m_iAction: 0:REBOOT 1:SHUTDOWN 2:POWEROFF 3:LOGOFF
int m_iAction = -1;

void BangShutDown(HWND hwndCaller, LPCSTR pszArgs);
void BangLogOff(HWND hwndCaller, LPCSTR pszArgs);
void BangRestart(HWND hwndCaller, LPCSTR pszArgs);

void BangCancel(HWND hwndCaller, LPCSTR pszArgs);

void Timer();
void ShutDown(UINT nID);

bool overrideRecycle = GetRCBoolDef("xRecycleOverride", false);
string beforeReycleAction = GetxRCLine("xRecycle", "Action", "");
void BangRecycle(HWND hwndCaller, LPCSTR pszArgs);

LPSHELLFOLDER2 m_pRecycleBin = NULL;
//----------------------------------------------------

HINSTANCE hInstance;
HWND messageHandler;

int lsMessages[] = {
	LM_GETREVID,
	0
};

LRESULT WINAPI MessageHandlerProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch(message)
	{
		case LM_GETREVID:
		{
			UINT uLength;
			StringCchPrintf((char*)lParam, 64, "%s.dll: %s", V_NAME, V_VERSION);
			
			if (SUCCEEDED(StringCchLength((char*)lParam, 64, &uLength)))
				return uLength;

			lParam = NULL;
			return 0;
		}

		case WM_TIMER:
		{
			switch( (int) wParam )
			{
				case TIMER_TRASHCHECK:
				{
					CheckTrash();

					break;
				}

				case TIMER_PRESHUTDOWN:
				{
					KillTimer(messageHandler, TIMER_PRESHUTDOWN);
					Timer();
					SetTimer(messageHandler, TIMER_SHUTDOWN, 60000, 0);

					break;
				}

				case TIMER_SHUTDOWN:
				{
					Timer();

					break;
				}

				case TIMER_RECYCLE:
				{
					KillTimer(messageHandler, TIMER_RECYCLE);

					PostMessage(GetLitestepWnd(), LM_RECYCLE, LR_RECYCLE, 0);
		
					break;
				}

				default:
				{
					break;
				}
			}
		}
	}

	return DefWindowProc(hWnd, message, wParam, lParam);
}

BOOL __stdcall _DllMainCRTStartup(HINSTANCE hInst, DWORD fdwReason, LPVOID lpvRes)
{
	// We don't need thread notifications for what we're doing.  Thus, get
	// rid of them, thereby eliminating some of the overhead of this DLL
	DisableThreadLibraryCalls(hInst);

	return TRUE;
}

int initModuleEx(HWND hParent, HINSTANCE hInstance, const char *lsPath)
{
	WNDCLASSEX wc;

	wc.cbSize = sizeof(WNDCLASSEX);
	wc.style = CS_GLOBALCLASS;
	wc.lpfnWndProc = MessageHandlerProc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hInstance = hInstance;
	wc.hbrBackground = 0;
	wc.hCursor = 0;
	wc.hIcon = 0;
	wc.lpszMenuName = 0;
	wc.lpszClassName = "xInterfaceMessageHandler";
	wc.hIconSm = 0;

	RegisterClassEx(&wc);

	messageHandler = CreateWindowEx(WS_EX_TOOLWINDOW,
		"xInterfaceMessageHandler",
		0,
		WS_POPUP,
		0, 0, 0, 0, 
		0,
		0,
		hInstance,
		0);

	if (!messageHandler)
		return 1;

	SendMessage(GetLitestepWnd(),
		LM_REGISTERMESSAGE,
		(WPARAM) messageHandler,
		(LPARAM) lsMessages);

	::hInstance = hInstance;

	AddBangCommand("!xTrashEmpty", BangEmptyTrash);

	AddBangCommand("!xShutDown", BangShutDown);
	AddBangCommand("!xLogoff", BangLogOff);
	AddBangCommand("!xRestart", BangRestart);

	AddBangCommand("!xCancel", BangCancel);

	if ( overrideRecycle )
		AddBangCommand("!Recycle", BangRecycle);
	else
		AddBangCommand("!xRecycle", BangRecycle);

	isTrashSizeAvailable = GetRecycleFolder();

	//True gives number of harddisks
	PrefixedLSSetVariable("xHardDisk", "Count", IntToString( GetTrashCount(true) ));

	PrefixedLSSetVariable("xSystemAction", "Time", "Not Active");
	PrefixedLSSetVariable("xSystemAction", "TimeLeft", "Not Active");

	CheckTrash(true);
	SetTimer(messageHandler, TIMER_TRASHCHECK, TrashTimeout*1000, 0);

	return 0;
}

void quitModule(HINSTANCE hInstance)
{
	KillTimer(messageHandler, TIMER_TRASHCHECK);
	KillTimer(messageHandler, TIMER_PRESHUTDOWN);
	KillTimer(messageHandler, TIMER_SHUTDOWN);
	KillTimer(messageHandler, TIMER_RECYCLE);

	if (m_pRecycleBin != NULL)
	{
		m_pRecycleBin->Release();
	}

	RemoveBangCommand("!xTrashEmpty");

	RemoveBangCommand("!xShutDown");
	RemoveBangCommand("!xLogoff");
	RemoveBangCommand("!xRestart");

	RemoveBangCommand("!xCancel");

	if ( !overrideRecycle )
		RemoveBangCommand("!xRecycle");

	SendMessage(GetLitestepWnd(),
		LM_UNREGISTERMESSAGE,
		(WPARAM) messageHandler,
		(LPARAM) lsMessages);

	DestroyWindow(messageHandler);

	UnregisterClass("xInterfaceMessageHandler", hInstance);
}


void CheckTrash(bool init)
{
	bool changed = false;

	int trashCount = GetTrashCount();

	if ( trashCount != oldtrashCount )
	{
		PrefixedLSSetVariable("xTrash", "ItemCount", IntToString(trashCount));
		changed = true;
	}

	string trashSize = "";

	if ( isTrashSizeAvailable )
		trashSize = GetTrashSize();

	if ( stricmp(trashSize.c_str(), oldtrashSize.c_str()) != NULL )
	{
		PrefixedLSSetVariable("xTrash", "ItemSize", trashSize);
		changed = true;	
	}

	if ( !BecomeEmpty.empty() && trashCount == 0 && (!isTrashEmpty || init) )
	{
		isTrashEmpty = true;
		LSExecute(NULL, BecomeEmpty.c_str(), SW_SHOWNORMAL);
		if ( !Change.empty() )
			LSExecute(NULL, Change.c_str(), SW_SHOWNORMAL);
	}
	else if ( !BecomeFilled.empty() && trashCount != 0 && (isTrashEmpty || init) )
	{
		isTrashEmpty = false;
		LSExecute(NULL, BecomeFilled.c_str(), SW_SHOWNORMAL);
		if ( !Change.empty() )
			LSExecute(NULL, Change.c_str(), SW_SHOWNORMAL);
	}
	else if ( !Change.empty() && changed )
	{
		LSExecute(NULL, Change.c_str(), SW_SHOWNORMAL);
	}

	oldtrashCount = trashCount;
	oldtrashSize = trashSize;
}

void BangEmptyTrash(HWND hwndCaller, LPCSTR pszArgs)
{
	SHEmptyRecycleBin( hwndCaller, NULL, /*SHERB_NOCONFIRMATION*/NULL );
}

// m_iAction: 0:REBOOT 1:SHUTDOWN 2:POWEROFF 3:LOGOFF
void BangShutDown(HWND hwndCaller, LPCSTR pszArgs)
{
	KillTimer(messageHandler, TIMER_SHUTDOWN);

	char mode[64], first[16], last[16];
	char *buffers[] = {mode, first, last};

	LCTokenize(pszArgs, buffers, 3, NULL);

	if ( stricmp(mode, ".delay") == NULL )
	{
		m_iHour = 0;
		m_iMinute = atoi(first);
		m_iSecond = atoi(last);
		m_bExactTime = false;
		m_iAction = 1;

		SetTimer(messageHandler, TIMER_SHUTDOWN, 1000, 0);
	}
	else if ( stricmp(mode, ".exact") == NULL )
	{
		m_iHour = atoi(first);
		m_iMinute = atoi(last);
		m_iSecond = 0;
		
		if ( m_iMinute < 10 )
			PrefixedLSSetVariable("xSystemAction", "Time", (IntToString(m_iHour) + ":0" + IntToString(m_iMinute)));
		else
			PrefixedLSSetVariable("xSystemAction", "Time", (IntToString(m_iHour) + ":" + IntToString(m_iMinute)));

		m_bExactTime = true;
		m_iAction = 1;

		CTime m_CTime = CTime::GetCurrentTime();

		int nHour = m_iHour-m_CTime.GetHour();
		int nMinute = m_iMinute-m_CTime.GetMinute()-1;
			
		if (nMinute < 0)
		{
			if (nHour > 0)
				nHour--;
			nMinute = 60-abs(nMinute);
		}

		if ( nMinute+1 < 10 )
			PrefixedLSSetVariable("xSystemAction", "TimeLeft", (IntToString(nHour) + ":0" + IntToString(nMinute+1)));
		else
			PrefixedLSSetVariable("xSystemAction", "TimeLeft", (IntToString(nHour) + ":" + IntToString(nMinute+1)));

		SetTimer(messageHandler, TIMER_PRESHUTDOWN, (max(0, 60-m_CTime.GetSecond())+1)*1000, 0);
	}
	else
	{
		ShutDown(1);
	}
}

// m_iAction: 0:REBOOT 1:SHUTDOWN 2:POWEROFF 3:LOGOFF
void BangLogOff(HWND hwndCaller, LPCSTR pszArgs)
{
	KillTimer(messageHandler, TIMER_SHUTDOWN);

	char mode[64], first[16], last[16];
	char *buffers[] = {mode, first, last};

	LCTokenize(pszArgs, buffers, 3, NULL);

	if ( stricmp(mode, ".delay") == NULL )
	{
		m_iHour = 0;
		m_iMinute = atoi(first);
		m_iSecond = atoi(last);
		m_bExactTime = false;
		m_iAction = 3;

		SetTimer(messageHandler, TIMER_SHUTDOWN, 1000, 0);
	}
	else if ( stricmp(mode, ".exact") == NULL )
	{
		m_iHour = atoi(first);
		m_iMinute = atoi(last);
		m_iSecond = 0;

		if ( m_iMinute < 10 )
			PrefixedLSSetVariable("xSystemAction", "Time", (IntToString(m_iHour) + ":0" + IntToString(m_iMinute)));
		else
			PrefixedLSSetVariable("xSystemAction", "Time", (IntToString(m_iHour) + ":" + IntToString(m_iMinute)));

		m_bExactTime = true;
		m_iAction = 3;
		
		CTime m_CTime = CTime::GetCurrentTime();

		int nHour = m_iHour-m_CTime.GetHour();
		int nMinute = m_iMinute-m_CTime.GetMinute()-1;
			
		if (nMinute < 0)
		{
			if (nHour > 0)
				nHour--;
			nMinute = 60-abs(nMinute);
		}

		if ( nMinute+1 < 10 )
			PrefixedLSSetVariable("xSystemAction", "TimeLeft", (IntToString(nHour) + ":0" + IntToString(nMinute+1)));
		else
			PrefixedLSSetVariable("xSystemAction", "TimeLeft", (IntToString(nHour) + ":" + IntToString(nMinute+1)));

		SetTimer(messageHandler, TIMER_PRESHUTDOWN, (max(0, 60-m_CTime.GetSecond())+1)*1000, 0);
	}
	else
	{
		ShutDown(3);
	}
}

// m_iAction: 0:REBOOT 1:SHUTDOWN 2:POWEROFF 3:LOGOFF
void BangRestart(HWND hwndCaller, LPCSTR pszArgs)
{
	KillTimer(messageHandler, TIMER_SHUTDOWN);

	char mode[64], first[16], last[16];
	char *buffers[] = {mode, first, last};

	LCTokenize(pszArgs, buffers, 3, NULL);

	if ( stricmp(mode, ".delay") == NULL )
	{
		m_iHour = 0;
		m_iMinute = atoi(first);
		m_iSecond = atoi(last);
		m_bExactTime = false;
		m_iAction = 0;

		SetTimer(messageHandler, TIMER_SHUTDOWN, 1000, 0);
	}
	else if ( stricmp(mode, ".exact") == NULL )
	{
		m_iHour = atoi(first);
		m_iMinute = atoi(last);
		m_iSecond = 0;
			
		if ( m_iMinute < 10 )
			PrefixedLSSetVariable("xSystemAction", "Time", (IntToString(m_iHour) + ":0" + IntToString(m_iMinute)));
		else
			PrefixedLSSetVariable("xSystemAction", "Time", (IntToString(m_iHour) + ":" + IntToString(m_iMinute)));

		m_bExactTime = true;
		m_iAction = 0;

		CTime m_CTime = CTime::GetCurrentTime();

		int nHour = m_iHour-m_CTime.GetHour();
		int nMinute = m_iMinute-m_CTime.GetMinute()-1;
			
		if (nMinute < 0)
		{
			if (nHour > 0)
				nHour--;
			nMinute = 60-abs(nMinute);
		}

		if ( nMinute+1 < 10 )
			PrefixedLSSetVariable("xSystemAction", "TimeLeft", (IntToString(nHour) + ":0" + IntToString(nMinute+1)));
		else
			PrefixedLSSetVariable("xSystemAction", "TimeLeft", (IntToString(nHour) + ":" + IntToString(nMinute+1)));

		SetTimer(messageHandler, TIMER_PRESHUTDOWN, (max(0, 60-m_CTime.GetSecond())+1)*1000, 0);
	}
	else
	{
		ShutDown(0);
	}
}

void BangCancel(HWND hwndCaller, LPCSTR pszArgs)
{
	KillTimer(messageHandler, TIMER_PRESHUTDOWN);
	KillTimer(messageHandler, TIMER_SHUTDOWN);
	m_iAction = -1;
	m_iHour = 0;
	m_iMinute = 0;
	m_iSecond = 0;

	PrefixedLSSetVariable("xSystemAction", "Time", "Not Active");
	PrefixedLSSetVariable("xSystemAction", "TimeLeft", "Not Active");

	m_bExactTime = false;
}

void ShutDown(UINT nID)
{	  
	HANDLE hToken;              // handle to process token 
	TOKEN_PRIVILEGES tkp;       // pointer to token structure 
       
	OpenProcessToken(GetCurrentProcess(), 
						TOKEN_ADJUST_PRIVILEGES | TOKEN_QUERY, &hToken) ;
			  
	// Get the LUID for shutdown privilege. 
			  
	LookupPrivilegeValue(NULL, SE_SHUTDOWN_NAME, 
							&tkp.Privileges[0].Luid); 
			  
	tkp.PrivilegeCount = 1;  // one privilege to set    
	tkp.Privileges[0].Attributes = SE_PRIVILEGE_ENABLED; 
			  
	// Get shutdown privilege for this process. 
	AdjustTokenPrivileges(hToken, FALSE, &tkp, 0, 
	  (PTOKEN_PRIVILEGES) NULL, 0); 
						  		  
	int nArgument;
			  
	switch(nID)
	{
		case 0:nArgument=EWX_REBOOT; break;
		case 1:nArgument=EWX_SHUTDOWN; break;
		case 2:nArgument=EWX_POWEROFF; break;
		case 3:nArgument=EWX_LOGOFF;
	}	  

	ExitWindowsEx(nArgument,0);
}

void Timer()
{
	int nHour;
	int nMinute;
     
	CTime m_CTime = CTime::GetCurrentTime();
			
	if (m_bExactTime)
	{
		nHour = m_iHour-m_CTime.GetHour();
		nMinute = m_iMinute-m_CTime.GetMinute()-1;
			
		if (nMinute < 0)
		{
			if (nHour > 0)
				nHour--;
			nMinute = 60-abs(nMinute);
		}
							
		if ( (m_CTime.GetHour() == m_iHour) && (m_CTime.GetMinute() == m_iMinute) )
		{
			KillTimer(messageHandler, TIMER_PRESHUTDOWN);
			KillTimer(messageHandler, TIMER_SHUTDOWN);
			ShutDown(m_iAction);
		}
		else
		{
			if ( nMinute+1 < 10 )
				PrefixedLSSetVariable("xSystemAction", "TimeLeft", (IntToString(nHour) + ":0" + IntToString(nMinute+1)));
			else
				PrefixedLSSetVariable("xSystemAction", "TimeLeft", (IntToString(nHour) + ":" + IntToString(nMinute+1)));
		}
	}
	else
	{
		if( (m_iMinute == 0) && (m_iSecond == 0))
		{
			KillTimer(messageHandler, TIMER_PRESHUTDOWN);
			KillTimer(messageHandler, TIMER_SHUTDOWN);
			ShutDown(m_iAction);
			return;
		}
				
		m_iSecond--;
		if (m_iSecond == 0)
		{
			if (m_iMinute > 0)
			{
				m_iMinute--;	
				m_iSecond = 59;
			}
		}		

		if ( m_iSecond < 10 )
			PrefixedLSSetVariable("xSystemAction", "TimeLeft", (IntToString(m_iMinute) + ":0" + IntToString(m_iSecond)));
		else
			PrefixedLSSetVariable("xSystemAction", "TimeLeft", (IntToString(m_iMinute) + ":" + IntToString(m_iSecond)));
	}	
}

void BangRecycle(HWND hwndCaller, LPCSTR pszArgs)
{	
	if ( !beforeReycleAction.empty() )
		LSExecute(NULL, beforeReycleAction.c_str(), SW_SHOWNORMAL);

	if ( stricmp( pszArgs, "" ) == NULL )
	{
		PostMessage(GetLitestepWnd(), LM_RECYCLE, LR_RECYCLE, 0);
	}
	else
	{
		SetTimer(messageHandler, TIMER_RECYCLE, atoi(pszArgs), 0);
	}
}